#include "rk.hh"

// Large functions from the rk namespace are implemented below
namespace rk {
    P4& P4::boost(const Boost& b)
    {
        if (m_ < 0.0)
        {
            const double msq(e_*e_ - p_.lengthSquared());
            assert(msq >= 0.0);
            msqIsNonNegative_ = true;
            m_ = sqrt(msq);
        }
        const double par(b.dir_.dot(p_));
        p_ += b.dir_*(b.c_*par - e_*b.s_);
        e_ = sqrt(m_*m_ + p_.lengthSquared()) * (e_ < 0.0 ? -1.0 : 1.0);
        return *this;
    }

    double lambda(const double x, const double y, const double z)
    {
        if (x < 0.0 || y < 0.0 || z < 0.0)
            assert(!"rk::lambda invalid arguments");
        else if (x == 0.0)
            return fabs(y - z);
        else if (y == 0.0)
            return fabs(x - z);
        else if (z == 0.0)
            return fabs(x - y);
        else
        {
            const double ymz(y - z);
            const double dtmp(ymz*ymz + x*(x - 2.0*(y + z)));
            assert(dtmp >= 0.0);
            return sqrt(dtmp);
        }
    }

    std::pair<P4,P4> phaseSpaceDecay(const P4& parent,
                                     const double m1,
                                     const double m2,
                                     const double rnd0,
                                     const double rnd1)
    {
        assert(m1 >= 0.0 && m2 >= 0.0);
        const double parentM(parent.m());
        assert(parentM >= m1 + m2);
        const Boost& lb = parent.labBoost();
        if (parentM == m1 + m2)
            return std::pair<P4,P4>(lb*P4(geom3::Vector3(), m1),
                                    lb*P4(geom3::Vector3(), m2));
        else
        {
            const geom3::UnitVector3& dir = 
                geom3::UnitVector3::random(rnd0, rnd1);
            const double p(lambda(parentM*parentM, m1*m1, m2*m2)/2.0/parentM);
            return std::pair<P4,P4>(lb*P4(dir*p, m1),
                                    lb*P4(dir*(-p), m2));            
        }
    }

    void phaseSpaceDecay(const P4& parent, const double m1, const double m2,
                         const double rnd0, const double rnd1,
                         P4* dau1, P4* dau2)
    {
        assert(dau1);
        assert(dau2);
        assert(m1 >= 0.0 && m2 >= 0.0);
        const double parentM(parent.m());
        assert(parentM >= m1 + m2);
        const Boost& lb = parent.labBoost();
        if (parentM == m1 + m2)
        {
            *dau1 = lb*P4(geom3::Vector3(), m1);
            *dau2 = lb*P4(geom3::Vector3(), m2);
        }
        else
        {
            const geom3::UnitVector3& dir = 
                geom3::UnitVector3::random(rnd0, rnd1);
            const double p(lambda(parentM*parentM, m1*m1, m2*m2)/2.0/parentM);
            *dau1 = lb*P4(dir*p, m1);
            *dau2 = lb*P4(dir*(-p), m2);
        }
    }
}
